#!/usr/bin/perl -w

use strict;

# sub POE::Kernel::TRACE_REFCNT () { 1 }

sub DEBUG () { 0 }

use Test::More tests => 16;

use POE;
use POE::Component::Generic;

my $N = 1;
if( $ENV{HARNESS_PERL_SWITCHES} ) {
    $N *= 5;
}

my $daemon=0;
#eval "use POE::Component::Daemon; \$daemon++";

my $generic = POE::Component::Generic->spawn( 
          package   => 't::P50',
          methods   => [ qw( new buildthing ) ],
          factories => [ qw( buildthing ) ],
          verbose   => 1,
          debug     => DEBUG
      );
      
my $C1 = 0;
my $C2 = 0;
my $PID = $$;

my $subobj1;
my $number1;
my $subobj2;

POE::Session->create(
    inline_states => {
        _start => sub {
            $poe_kernel->alias_set( 'worker' );
            diag( "$N seconds" );
            $poe_kernel->delay( 'buildthing', $N );
            if( $daemon ) {
                $poe_kernel->sig( USR1=>'USR1' );
            }
        },
        USR1 => sub { Daemon->__peek( 1 ); },

        _stop => sub {
            DEBUG and warn "_stop";
        },

        buildthing => sub {
            $generic->buildthing( { event=>'builtthing' }, 
                                    ( number=>17 ) );
        },
        builtthing => sub {
            my( $res, $obj ) = @_[ ARG0, ARG1 ];

            ok( (not $res->{error}), "No errors" )
                        or die "Error = $res->{error}";
            ok( $obj, "Got first object" ); 
            ok( $obj->ID, "Has an ID" );
            ok( ($obj->ID =~ /poegenericOBJ000000/), 
                    "ID looks like we expect" );

            $subobj1 = $obj;



            is( $obj->{package}, 'Duffus', "Nice package" );
            is_deeply( $obj->{package_map}, {new=>'Duffus', number=>'Duffus'},
                        "Nice package_map" );

            $generic->yield( buildthing => { event=>'builtthing2' }, 
                                    ( number=>42 ) );
        },
        builtthing2 => sub {
            my( $res, $obj ) = @_[ ARG0, ARG1 ];

            ok( (not $res->{error}), "No errors" )
                        or die "Error = $res->{error}";
            ok( $obj, "Got second object" ); 
            
            isnt( $obj, $subobj1, "Not the same objects" );
            isnt( $obj->ID, $subobj1->ID, "Not the same IDs" );
            
            $subobj2 = $obj;
            $subobj1->number( {event=>'number1'} );     ## psuedo-method
        },
        number1 => sub {
            my( $res, $number ) = @_[ ARG0, ARG1 ];
            ok( (not $res->{error}), "No errors" )
                        or die "Error = $res->{error}";

            is( $number, 17, "Got number 17" ); 
            $number1 = $number;
            $subobj2->yield( 'number', {event=>'number2'} );    ## yield
        },
        number2 => sub {
            my( $res, $number ) = @_[ ARG0, ARG1 ];
            ok( (not $res->{error}), "No errors" )
                        or die "Error = ", join "\n", @{$res->{errors}};

            isnt( $number, $number1, "Not the same number" );

            $subobj1->call( 'number', {event=>'number3'} );     ## call
        },
        number3 => sub {
            my( $res, $number ) = @_[ ARG0, ARG1 ];
            ok( (not $res->{error}), "No errors" )
                        or die "Error = ", join "\n", @{$res->{errors}};

            is( $number, $number1, "Got the same number" );

            $generic->shutdown;
        },

    },
);

$poe_kernel->run;

