#!/usr/bin/perl -w

use strict;

my $tests = 18;
use Test::More ( tests => 18 );
use POE;
use POE::Component::Generic;

sub DEBUG () { 0 }

my $daemon=0;
# eval "use POE::Component::Daemon; \$daemon++";

my $has_ssh=0;
eval "use POE::Component::Generic::Net::SSH2; \$has_ssh++";
warn $@ if $@;

my $N = 1;
if( $ENV{HARNESS_PERL_SWITCHES} ) {
    $N *= 5;
}


##########################################
use t::Config;
my $conf = $t::Config::VAR1->{ssh};

SKIP: {

    skip "SSH not configured by Makefile.PL", $tests
            unless $conf;

    unless( $has_ssh ) {
        $conf = 0;
        skip "Net::SSH::Perl not installed",  $tests;
    }


    unless( $conf->{password} ) {
        $conf = 0;
        skip "No password for SSH", $tests;
    }

}

exit 0 unless $conf;


##########################################
my $ssh = POE::Component::Generic::Net::SSH2->spawn(
                alias   => 'my-ssh',
                verbose => 1,
                debug   => DEBUG,
            );

my $channel;
my $stdout;

##########################################
POE::Session->create( 
    inline_states => {
        _start => sub {
            $poe_kernel->alias_set( 'worker' );
            diag( "$N seconds" );
            $poe_kernel->delay( 'connect', $N );
            if( $daemon ) {
                $poe_kernel->sig( USR1=>'USR1' );
            }
        },
        USR1 => sub { Daemon->__peek( 1 ); },

        _stop => sub {
            DEBUG and warn "_stop";
        },

        ###############################
        connect => sub {
            $ssh->connect( {event=>'connected'}, 
                            $conf->{host}, $conf->{port} );
        },
        connected => sub {
            $ssh->auth_password( {event=>'login'}, 
                                     $conf->{user}, $conf->{password} );
        },
        error => sub {
            my( $resp, $code, $name, $error ) = @_[ARG0, $#_];
            die "Error $name ($code) $error";
        },

        ###########
        login => sub {
            my( $resp, $OK ) = @_[ARG0..$#_];
            ok( !$resp->{error}, "No error" ) or die "Error: $resp->{error}";
            ok( $OK, "Logged in" ) or do {
                $ssh->error( {event=>'error', wantarray=>1} );
                return;
            };
            $ssh->auth_ok( {event=>'authed'} );
            return;
        },
        authed => sub {
            my( $resp, $authed ) = @_[ARG0..$#_];
            ok( !$resp->{error}, "No error" ) or die "Error: $resp->{error}";
            ok( $authed, "Logged in" ) or die "Not logged in?";

            $ssh->channel( {event=>'got_channel'} );
            return;
        },

        ###########
        got_channel => sub {
            my( $resp, $ch ) = @_[ ARG0..$#_ ];
            ok( !$resp->{error}, "No error" ) or die "Error: $resp->{error}";

            ok( $ch, "Got a channel" );
            $channel = $ch;

            $channel->call( 'cmd', { event=>'output', 
                                     wantarray=>1 }, "ls -l" );
        },
        output => sub {
            my( $resp, $stdout ) = @_[ARG0..$#_];
            ok( !$resp->{error}, "No error on exec" ) 
                                    or die "Error: $resp->{error}";

            ok( ($stdout =~ /\d+ $conf->{user} $conf->{user} \d+/), 
                    "Output of ls looks ls-like" )
                                    or die "Output=$stdout";

            undef( $channel );

            $ssh->channel( {event=>'got_channel2'} );
            return;
        },

        ###########
        got_channel2 => sub {
            my( $resp, $ch ) = @_[ ARG0..$#_ ];
            ok( !$resp->{error}, "No error" ) or die "Error: $resp->{error}";

            ok( $ch, "Got a channel" );
            $channel = $ch;

            $channel->call( 'cmd', { event=>'output2', 
                                     wantarray=>1 }, "some-error" );
        },
        output2 => sub {
            my( $resp, $stdout, $stderr ) = @_[ARG0..$#_];
            ok( !$resp->{error}, "No error on exec" ) 
                                    or die "Error: $resp->{error}";

            ok( !$stdout, "No output" );
            ok( ($stderr =~ /some-error/), "Expected error message" )
                or warn "STDERR=$stderr";

            undef( $channel );
            # $ssh->shutdown();
            $ssh->channel( {event=>'setup_handlers'} );
        },

        
        ###########
        setup_handlers => sub {
            my( $resp, $ch ) = @_[ ARG0..$#_ ];
            ok( !$resp->{error}, "No error" ) or die "Error: $resp->{error}";

            ok( $ch, "Got a channel" );
            $channel = $ch;
            $stdout = '';

            $channel->handler_stderr( {}, 'h_stderr' );
            $channel->handler_stdout( {}, 'h_stdout' );
            $channel->handler_closed( {}, 'h_closed' );

            $channel->exec( {}, 'ls -l' );
        },

        h_stderr => sub {
            my( $text, $bytes ) = @_[ARG0..$#_];
            die "STDERR";
            $ssh->shutdown;    
            return;
        },
        h_stdout => sub {
            my( $text, $bytes ) = @_[ARG0..$#_];
            # warn "STDOUT: $text";
            $stdout .= $text;
            return;
        },
        h_closed => sub {
            DEBUG and warn "CLOSED";

            ok( $stdout, "Got some text from the command" );
            ok( ($stdout =~ /\d+ $conf->{user} $conf->{user} \d+/), 
                    "Output of ls looks ls-like" )
                                    or die "Output=$stdout";
            
            
            diag( "kill -USR1 $$ to see why this doesn't exit" );
            undef( $channel );
            $ssh->shutdown;    
            return;
        },
    }    
);


$poe_kernel->run;

pass( "Sane exit" );

